dofile("Scorpion.rte/Customization.lua");

-----------------------------------fake offset rotation code-
-------------------------------------------------------------

function scrotate(oldvector,offset,angle)
	local newvector = Vector(0,0);
	newvector.X = oldvector.X + (offset.X * math.cos(angle) + offset.Y * math.sin(angle));
	newvector.Y = oldvector.Y + (offset.X * -math.sin(angle) + offset.Y * math.cos(angle));
	return newvector;
end

--------------------------------------flight child functions-
-------------------------------------------------------------

--flight functions are stripped so other
--systems can manage through them
function sctiltright(pointer)
	pointer.AoA = pointer.AoA - (math.pi / scorpionangleofattackrate);
end

function sctiltleft(pointer)
	pointer.AoA = pointer.AoA + (math.pi / scorpionangleofattackrate);
end

function scthrottleup(pointer)
	pointer.throttle = pointer.throttle + scorpionthrottlerate;
end

function scthrottledown(pointer)
	pointer.throttle = pointer.throttle - scorpionthrottlerate;
end

--------------------------------------flight parent function-
-------------------------------------------------------------

function scf(pointer)
	local controller = pointer:GetController();
	if controller:IsState(Controller.MOVE_UP) then
		if pointer.throttle < scorpionmaxthrottle then
			scthrottleup(pointer);
		end
	elseif controller:IsState(Controller.MOVE_DOWN) then
		if pointer.throttle > scorpionminthrottle then
			scthrottledown(pointer);
		end
	end
	if controller:IsState(Controller.MOVE_RIGHT) then
		if pointer.HFlipped == true then
			if pointer.AoA > math.pi / 4 then
				sctiltright(pointer)
			end
		else
			if pointer.AoA > 0 then
				sctiltright(pointer)
			end
		end
	elseif controller:IsState(Controller.MOVE_LEFT) then
		if pointer.HFlipped == true then
			if pointer.AoA < math.pi then
				sctiltleft(pointer)
			end
		else
			if pointer.AoA < math.pi - (math.pi / 4) then
				sctiltleft(pointer)
			end
		end
	end
end

--------------------------------------cannon child functions-
-------------------------------------------------------------

--these two functions are nonspecific so that
--other control systems can manage them
--same deal for flight
function scaimup(pointer)
	pointer.aimangle = pointer.aimangle + math.pi / scorpionaimanglerate;
end

function scaimdown(pointer)
	pointer.aimangle = pointer.aimangle - math.pi / scorpionaimanglerate;
end
--fires the actual cannon shot
function sccannon(pointer)
	if pointer.guntimer:IsPastSimMS(scorpioncannonrateoffire) == true then
		pointer.guntimer:Reset();
		local shot = CreateMOSRotating("Cannon Shot","Scorpion.rte");
		local gunangle = pointer.RotAngle + pointer.aimangle;
		local muzzlepos = Vector(0,0);
		local spos = Vector(0,0);
		if pointer.HFlipped == false then
			local moffset = Vector(scorpioncannonoffsetx,scorpioncannonoffsety);
			muzzlepos = scrotate(pointer.Pos,moffset,pointer.RotAngle);
			local soffset = Vector(50,0);
			spos = scrotate(muzzlepos,soffset,gunangle);
			shot.Vel = Vector(100 * math.cos(-gunangle),100 * math.sin(-gunangle));
		else
			local moffset = Vector(-scorpioncannonoffsetx,scorpioncannonoffsety);
			muzzlepos = scrotate(pointer.Pos,moffset,pointer.RotAngle);
			local soffset = Vector(-50,0);
			spos = scrotate(muzzlepos,soffset,gunangle);
			shot.Vel = Vector(-100 * math.cos(-gunangle),-100 * math.sin(-gunangle));
		end
		shot.Pos = spos;
		shot.RotAngle = gunangle;
		shot.HFlipped = pointer.HFlipped;
		MovableMan:AddParticle(shot);
	end
end
--moves barrel of gun up/down while hovering
function scaimgun(pointer)
	if pointer.HFlipped == true then
		if pointer:GetController():IsState(Controller.MOVE_DOWN) then
			if pointer.aimangle < math.pi / 2 then
				scaimup(pointer);
			end
		elseif pointer:GetController():IsState(Controller.MOVE_UP) then
			if pointer.aimangle > 0 then
				scaimdown(pointer);
			end
		end
	else
		if pointer:GetController():IsState(Controller.MOVE_UP) then
			if pointer.aimangle < 0 then
				scaimup(pointer);
			end
		elseif pointer:GetController():IsState(Controller.MOVE_DOWN) then
			if pointer.aimangle > -math.pi / 2 then
				scaimdown(pointer);
			end
		end
	end
end

---------------------------------------rocket child function-
-------------------------------------------------------------

function scrockets(pointer)
	if pointer.rockettimer:IsPastSimMS(scorpionrocketrate) == true then
		pointer.rockettimer:Reset();
		local xoffset = 20;
		local rocket = CreateMOSRotating("Test Rocket");
		if pointer.HFlipped == true then
			xoffset = -20;
		end
		local launcher = math.random(1,3);
		if launcher == 1 then
			rocket.Pos = Vector(pointer.Pos.X - (xoffset * 2),pointer.Pos.Y - 25);
		elseif launcher == 2 then
			rocket.Pos = Vector(pointer.Pos.X - (xoffset * 3),pointer.Pos.Y - 25);
		else
			rocket.Pos = Vector(pointer.Pos.X - (xoffset * 4),pointer.Pos.Y - 25);
		end
		local speed = math.random(30,35)
		local dir = math.random(55,70);
		if pointer.HFlipped == true then
			rocket.Vel.X = speed * math.cos(math.pi + dir / 100);
			rocket.Vel.Y = speed * math.sin(math.pi + dir / 100);
		else
			rocket.Vel.X = speed * math.cos(-(dir / 100));
			rocket.Vel.Y = speed * math.sin(-(dir / 100));
		end
		rocket.RotAngle = dir;
		MovableMan:AddParticle(rocket);
	end
end

--------------------------------------weapon parent function-
-------------------------------------------------------------

function scw(pointer)
	local controller = pointer:GetController();

	if controller:IsState(Controller.PRIMARY_ACTION) then
		sccannon(pointer);
	end
	if pointer.hover == 1 then
		if controller:IsState(Controller.BODY_CROUCH) then
			if pointer.shottimer:IsPastSimMS(2000) == true then
				pointer.shottimer:Reset();
				pointer.shootrockets = true;
			end
		end
	end
	scaimgun(pointer);
end

----------------------------------------------animation code-
-------------------------------------------------------------
function scanim(pointer)
	--animation: manage pseudoattachable position, rotation, etc)
	--"fan" is just the shortname for the engine used
	--this script keeps it at the right position and
	--rotation
	local fan = pointer.fan;
	fan.RotAngle = pointer.AoA;
	local foffset = Vector(0,0);
	if pointer.HFlipped == false then
		foffset = Vector(scorpionfanoffsetx,scorpionfanoffsety);
	else
		foffset = Vector(-scorpionfanoffsetx,scorpionfanoffsety);
	end
	fan.Pos = scrotate(pointer.Pos,foffset,pointer.RotAngle);
	fan.Vel = pointer.Vel;

	--the gun is just that. this keeps it offset and
	--rotated properly
	local gun = pointer.gun;
	gun.RotAngle = pointer.RotAngle + pointer.aimangle;
	local goffset = Vector(0,0);
	if pointer.HFlipped == false then
		goffset = Vector(scorpioncannonoffsetx,scorpioncannonoffsety);
	else
		goffset = Vector(-scorpioncannonoffsetx,scorpioncannonoffsety);
	end
	local GunOffset = pointer.Pos + pointer:RotateOffset(Vector(scorpioncannonoffsetx,scorpioncannonoffsety))
	gun.Pos = GunOffset; --scrotate(pointer.Pos,goffset,pointer.RotAngle);
	gun.HFlipped = pointer.HFlipped;
	gun.Vel = pointer.Vel;
end

--------------------------------------------------hover code-
-------------------------------------------------------------

--the "hover" script works to keep the vehicle stationary
--does so by balancing throttle and killing fan rotation
function schover(pointer)
	if pointer.throttle >= (SceneMan.GlobalAcc.Y / 2) then
		scthrottledown(pointer);
	elseif pointer.throttle <= (SceneMan.GlobalAcc.Y / 2) - 0.2 then
		scthrottleup(pointer);
	elseif pointer.hashoverpoint == false then
		pointer.hashoverpoint = true;
		pointer.hoverpoint = pointer.Pos.Y;
	end
	if pointer.AoA < math.pi / 2 then
		sctiltleft(pointer);
	elseif pointer.AoA > math.pi / 2 then
		sctiltright(pointer);
	end
end

function scturbulence(pointer)
	local hpoint = pointer.hoverpoint;
	if pointer.turbtimer:IsPastSimMS(math.random(50,2000)) == true then
		pointer.turbtimer:Reset();
		if (pointer.Pos.X % hpoint.X) < 12 then
			pointer.Pos.X = pointer.Pos.X + math.random(-12,12);
		end
		if (pointer.Pos.Y % hpoint.Y) < 12 then
			pointer.Pos.Y = pointer.Pos.Y + math.random(-12,12);
		end
	end
	--[[
	if pointer.offset == Vector(0,0) then
		if pointer.turbtimer:IsPastSimMS(math.random(4000,8000)) == true then
			pointer.turbtimer:Reset();
			pointer.turbulent = true;
			pointer.offset = Vector(math.random(9,14),math.random(math.pi * 0.4, math.pi * 0.6));
		end
	else
		if pointer.turbtimer:IsPastSimMS(math.random(100,1000)) == true then
			pointer.turbulent = false;
			pointer.offset = Vector(0,0);
		end
	end
	if pointer.turbulent == true then
		if pointer.throttle > pointer.offset.X then
			scthrottledown(pointer);
			scthrottledown(pointer);
		elseif pointer.throttle < pointer.offset.X then
			scthrottleup(pointer);
			scthrottleup(pointer);
		end
		if pointer.AoA > pointer.offset.Y then
			sctiltright(pointer);
			sctiltright(pointer);
		elseif pointer.AoA < pointer.offset.Y then
			sctiltleft(pointer);
			sctiltleft(pointer);
		end
	end
	]]--
end

---------------------------------------------------------------------------------------------	local GunOffset = self.Pos + self:RotateOffset(Vector(32, 20))

function scswitch(pointer)
	if pointer.switchtimer:IsPastSimMS(500) == true then
		if pointer:GetController():IsState(Controller.SECONDARY_ACTION) == true then
			pointer.switchtimer:Reset();
			if pointer.hover == 1 then
				if pointer.shootrockets == false then
					pointer.hover = 0;
				end
			elseif pointer.hover == 0 then
				pointer.hover = 1;
				pointer.shottimer:Reset();
			end
		end
	end
end

function schflip(pointer)
	if pointer.HFlipped == true then
		if pointer.AoA < math.pi / 2 and pointer:GetController():IsState(Controller.MOVE_RIGHT) == true then
			pointer.HFlipped = false;
		end
	else
		if pointer.AoA > math.pi / 2 and pointer:GetController():IsState(Controller.MOVE_LEFT) == true then
			pointer.HFlipped = true;
		end
	end
end

function sccrash(pointer)
	if pointer.Health < 0 then
		pointer:GibThis();
	end
end

function Create(self)
	self.fan = CreateMOSRotating("Scorpion Engine","Scorpion.rte");
	MovableMan:AddParticle(self.fan);
	self.gun = CreateMOSRotating("Scorpion Cannon","Scorpion.rte");
	MovableMan:AddParticle(self.gun);

	self.shottimer,self.rockettimer,self.switchtimer,self.guntimer,self.turbtimer = Timer(),Timer(),Timer(),Timer(),Timer();

	self.shootrockets = false;
	self.aimangle,self.hover = 0,0;

	self.throttle = 6;
	self.AoA = math.pi / 2;
	self.hoverpoint = Vector(0,0);
	self.hashoverpoint = false;
end

function Update(self)
	--autorun functions:
	--animation of engine, gun
	scanim(self);
	--switching hover on/off
	scswitch(self);
	--horizontally flip
	schflip(self);
	--keep it from raping ground when it crashes
	sccrash(self);

	self:ClearForces();
	self:ClearImpulseForces();
	
	self.AngVel = 0;
	self.fan.Age = 1;

	local fanforce = Vector((math.cos(-self.AoA) * self.throttle),(math.sin(-self.AoA) * self.throttle));
	local finalvel = Vector(fanforce.X,fanforce.Y + (SceneMan.GlobalAcc.Y / 2));
	self.Vel = finalvel;
	
	self.RotAngle = -1 * (self.Vel.X / 50);
	
	if self.shootrockets == false then
		scw(self)
	elseif self.shootrockets == true then
		if self.shottimer:IsPastSimMS(scorpionrocketbarragelength) == true then
			self.shottimer:Reset();
			self.shootrockets = false;
		else
			scrockets(self)
		end
	end

	if self.hover == 0 then
		self.hashoverpoint = false;
		scf(self);
		if self.aimangle > 0 then
			scaimdown(self);
		elseif self.aimangle < 0 then
			scaimup(self);
		end
	else
		schover(self);
	end

	if ActivityMan:GetActivity().ActivityState == Activity.EDITING then
		self.hover = 1;
	else
		if self.Age > 1500 then
			if self:GetController().Player == -1 then
				self.hover = 1;
			end
		end
	end
end

function Destroy(self)
	if MovableMan:IsParticle(self.fan) == true then
		self.fan.ToDelete = true;
	end
	if MovableMan:IsParticle(self.gun) == true then
		self.gun.ToDelete = true;
	end
end